"
WriteStream / WriteStreamPortable is an ANSI compliant implementation of
 WriteStream, a PositionableStream that allows its objects to be written,
 but not read.

--- instVar writeLimit
A SmallInteger; the last position in the collection available for writing.
"
Class {
	#name : 'WriteStream',
	#superclass : 'PositionableStream',
	#instVars : [
		'writeLimit'
	],
	#classVars : [
		'Cr',
		'CrLf',
		'CrTab'
	],
	#category : nil
}

{ #category : 'Instance Creation' }
WriteStream class >> new [

"Disallowed.  To create a new WriteStream, use the class method on: instead."

self shouldNotImplement: #new

]

{ #category : 'Instance Creation' }
WriteStream class >> on: aCollection from: firstIndex to: lastIndex [
"Answer an instance of me on a copy of the argument, aCollection,
determined by the indices firstIndex and lastIndex. Position the instance
at the beginning of the collection."

^self basicNew
	on: aCollection
	from: firstIndex
	to: lastIndex

]

{ #category : 'Instance Creation' }
WriteStream class >> with: aCollection [
"Answer an instance of me on the argument, aCollection, positioned to
store objects at the end of aCollection."

^self basicNew with: aCollection

]

{ #category : 'Instance Creation' }
WriteStream class >> with: aCollection from: firstIndex to: lastIndex [
"Answer an instance of me on the subcollection of the argument,
aCollection, determined by the indices firstIndex and lastIndex. Position
the instance to store at the end of the subcollection."

^self basicNew with: (aCollection copyFrom: firstIndex to: lastIndex)

]

{ #category : 'Adding' }
WriteStream >> _nextPut: anObject [
"see PrintStream>>_nextPut:"

self nextPut: anObject

]

{ #category : 'Accessing' }
WriteStream >> contents [
"WriteStreams return the portion of their collection that has been written:
 the collection up to the next write-position."

readLimit := readLimit max: position.
^collection copyFrom: 1 to: position

]

{ #category : 'Character writing' }
WriteStream >> cr [
"Append a return character to the receiver."

self nextPut:  Cr   .

]

{ #category : 'Character writing' }
WriteStream >> crlf [
"Append a carriage return character followed by a line feed character to the receiver."

self nextPutAll: CrLf .

]

{ #category : 'Character writing' }
WriteStream >> crtab [
"Append a return character, followed by a single tab character, to the
receiver."

self nextPutAll: CrTab .

]

{ #category : 'Character writing' }
WriteStream >> crtab: anInteger [
"Append a return character, followed by anInteger tab characters, to the
receiver."

self nextPut: Cr .
anInteger timesRepeat: [self nextPut: Tab  ]

]

{ #category : 'ANSI' }
WriteStream >> flush [
	"Update a stream's backing store.
	Upon return, if the receiver is a write-back stream, the state of the
	stream backing store must be consistent with the current state of the
	receiver.
	If the receiver is not a write-back stream, the effect of this
	message is unspecified."

	"We do nothing; this method is provided for ANSI compatibility"

]

{ #category : 'Accessing' }
WriteStream >> next [

"Disallowed.  You cannot read a WriteStream."

self shouldNotImplement: #next

]

{ #category : 'Adding' }
WriteStream >> next: anInteger putAll: aCollection startingAt: startIndex [
"Store the next anInteger elements from the given collection."

| newEnd numPut |
collection class == aCollection class ifFalse:
	[^ super next: anInteger putAll: aCollection startingAt: startIndex ].

numPut := anInteger min: (aCollection size - startIndex + 1).
newEnd := position + numPut.
newEnd > writeLimit ifTrue:
	[^ super next: anInteger putAll: aCollection startingAt: startIndex "Trigger normal pastEndPut: logic"].

collection replaceFrom: position+1 to: newEnd with: aCollection startingAt: startIndex.
position := newEnd.

]

{ #category : 'Adding' }
WriteStream >> nextPut: anObject [

"Inserts anObject as the next element that the receiver can access for writing.
 Returns anObject."

position >= writeLimit
	ifTrue: [^ self pastEndPut: anObject]
	ifFalse:[ | res |
		res := collection at: position + 1 put: anObject.
		position := position + 1.
		^res]

]

{ #category : 'Adding' }
WriteStream >> nextPutAll: aCollection [

"Inserts the elements of aCollection as the next elements that the receiver can
 access.  Returns aCollection."


 | newEnd |
 collection class == aCollection class ifFalse:
 	[^ super nextPutAll: aCollection ].

 newEnd := position + aCollection size.
 collection replaceFrom: position+1 to: newEnd  with: aCollection startingAt: 1.

 newEnd > writeLimit
 	ifTrue: [
  	 	collection size: newEnd.
 	 	writeLimit := newEnd ].
 position := newEnd.

^ aCollection

]

{ #category : 'Adding' }
WriteStream >> nextPutAllBytes: aCharacterCollection [
  "Inserts the byte contents of aCharacterCollection as the next elements
 that the receiver can access.  Returns aCollection.  The receiver's collection
 must be a type of String."

  "Used in the implementation of PassiveObject."

  position == collection size
    ifFalse: [ ^ super nextPutAllBytes: aCharacterCollection ].
  collection addAllBytes: aCharacterCollection.
  position := position + aCharacterCollection _basicSize.
  ^ aCharacterCollection

]

{ #category : 'Private' }
WriteStream >> on: aCollection [

super on: aCollection.
readLimit := 0.
writeLimit := aCollection size

]

{ #category : 'Private' }
WriteStream >> on: aCollection from: firstIndex to: lastIndex [

| len |
collection := aCollection.
readLimit := writeLimit := lastIndex > (len := collection size)
	ifTrue: [len]
	ifFalse: [lastIndex].
position := firstIndex <= 1
	ifTrue: [0]
	ifFalse: [firstIndex - 1]

]

{ #category : 'Private' }
WriteStream >> pastEndPut: anObject [
"Grow the collection by adding anObject one past end.
Then put <anObject> at the current write position."

collection at: position + 1 put: anObject.
position := position + 1.
writeLimit := collection size.
^ anObject

]

{ #category : 'Positioning' }
WriteStream >> position: anInteger [
"Refer to the comment in PositionableStream|position:."

readLimit := readLimit max: position.
super position: anInteger

]

{ #category : 'Adding' }
WriteStream >> print: anObject [

	anObject printOn: self.

]

{ #category : 'Positioning' }
WriteStream >> reset [
"Refer to the comment in PositionableStream|reset."

readLimit := readLimit max: position.
position := 0

]

{ #category : 'Positioning' }
WriteStream >> setToEnd [
"Refer to the comment in PositionableStream|setToEnd."

readLimit := readLimit max: position.
super setToEnd.

]

{ #category : 'Positioning' }
WriteStream >> size [

^readLimit := readLimit max: position

]

{ #category : 'Accessing' }
WriteStream >> skip: anInteger [
"Set the receiver's position to be the current position+anInteger. Do not
 throw error if skipAmount would exceed collection bounds - ANSI compliance. "

self position: ((position + anInteger max: 0) min: writeLimit)

]

{ #category : 'Character writing' }
WriteStream >> space: anInteger [
"Append anInteger space characters to the receiver."

anInteger timesRepeat: [self nextPut: $    ]

]

{ #category : 'Character writing' }
WriteStream >> tab: anInteger [
"Append anInteger tab characters to the receiver."

anInteger timesRepeat: [ self nextPut: Tab ]

]

{ #category : 'Private' }
WriteStream >> with: aCollection [

super on: aCollection.
position := readLimit := writeLimit := aCollection size

]
